controllers.controller('WallDateCtrl', function($scope,
                                                 $user,
                                                 $process,
                                                 $vkfilter,
                                                 $state,
                                                 $langControllerService){
  var $lang = $langControllerService.$lang;

  var edit = $lang.edit;

  var dateOptions = {
    date: new Date(),
    mode: 'date'
  };

  var timeOptions = {
    date: new Date(),
    is24Hour : true,
    mode: 'time'
  };

  function onDateSuccess(date) {
    $scope.inputs.date.value = date;
    $scope.inputs.date.text = $scope.toFullDate(date).show;
    $scope.$apply();
  }

  function onTimeSuccess(time) {
    $scope.inputs.time.value = time;
    $scope.inputs.time.text = $scope.toFullTime(time);
    $scope.$apply();
  }

  function onFromTimeSuccess(time) {
    $scope.inputs.period.from_time.value = time;
    $scope.inputs.period.from_time.text = $scope.toFullTime(time);
    $scope.$apply();
  }

  function onToTimeSuccess(time) {
    $scope.inputs.period.to_time.value = time;
    $scope.inputs.period.to_time.text = $scope.toFullTime(time);
    $scope.$apply();
  }

  function onFromDateSuccess(date) {
    $scope.inputs.period.from_date.value = date;
    $scope.inputs.period.from_date.text = $scope.toFullDate(date).show;
    $scope.$apply();
  }

  function onToDateSuccess(date) {
    $scope.inputs.period.to_date.value = date;
    $scope.inputs.period.to_date.text = $scope.toFullDate(date).show;
    $scope.$apply();
  }

  $scope.showDate = function() {
    datePicker.show(dateOptions, onDateSuccess);
  };

  $scope.showFromDate = function() {
    datePicker.show(dateOptions, onFromDateSuccess);
  };

  $scope.showToDate = function() {
    datePicker.show(dateOptions, onToDateSuccess);
  };

  $scope.showTime = function () {
    datePicker.show(timeOptions, onTimeSuccess)
  };

  $scope.showFromTime = function () {
    datePicker.show(timeOptions, onFromTimeSuccess)
  };

  $scope.showToTime = function () {
    datePicker.show(timeOptions, onToTimeSuccess)
  };

  $scope.inputs = {
    date : {
      disabled : true,
      text : edit
    },
    time : {
      disabled : true,
      text : edit
    },
    period : {
      setDisabledValue : function (value) {
        var _this = this;
        for (input in _this) {
          if (typeof(_this[input]) === 'object') {
            _this[input].disabled = value;
          }
        }
      },
      valuesCorrect : function(){
        var _this = this;
        for (input in _this) {
          if (typeof(_this[input]) === 'object') {
            if (!_this[input].value){
              return false;
            }
          }
        }
        return true;
      },
      from_time : {
        disabled : true,
        text : edit
      },
      from_date : {
        disabled : true,
        text : edit
      },
      to_time : {
        disabled : true,
        text : edit
      },
      to_date : {
        disabled : true,
        text : edit
      }
    },
    apply : {
      disabled : true
    }
  };

  $scope.title = $lang.wall.delete_date;
  $scope.desc = $lang.wall.delete_date_desc;
  $scope.apply_btn_text = $lang.apply;
  $scope.delete_older = $lang.wall.delete_older;
  $scope.delete_younger = $lang.wall.delete_younger;
  $scope.delete_period = $lang.wall.delete_period;
  $scope.date = $lang.wall.date;
  $scope.time = $lang.wall.time;
  $scope.from = $lang.wall.from;
  $scope.to = $lang.wall.to;

  $scope.checkBoxes = {
    filters : {
      date: {
        value: false,
        any_checked : function() {
          for (var filter in $scope.checkBoxes.filters) {
            if (filter != "date") {
              if ($scope.checkBoxes.filters[filter].value) {
                switch (filter) {
                  case 'older' :
                  case 'younger' :
                    $scope.inputs.date.disabled = false;
                    $scope.inputs.time.disabled = false;
                    break;
                  case 'period' :
                    $scope.inputs.period.setDisabledValue(false);
                    break;
                }
                return true;
              }
            }
          }
          return false;
        },
        method: function () {
          if (this.value) {
            $scope.inputs.apply.disabled = !this.any_checked();
          } else {
           $scope.inputs.apply.disabled = true;
           $scope.inputs.period.setDisabledValue(true);
           $scope.inputs.date.disabled = true;
           $scope.inputs.time.disabled = true;
          }
          $scope.checkBoxes.filters.younger.disabled = !this.value;
          $scope.checkBoxes.filters.older.disabled = !this.value;
          $scope.checkBoxes.filters.period.disabled = !this.value;
        }
      },
      older : {
        value : false,
        disabled : false,
        method : function() {
          if (this.value) {
            $scope.checkBoxes.filters.younger.value = false;
            $scope.checkBoxes.filters.period.value = false;
            $scope.inputs.date.disabled = false;
            $scope.inputs.time.disabled = false;
            $scope.inputs.period.setDisabledValue(true);
          }
          $scope.inputs.apply.disabled = this.value ? false : true;
        }
      },
      younger : {
        value : false,
        disabled : false,
        method : function() {
          if (this.value) {
            $scope.checkBoxes.filters.older.value = false;
            $scope.checkBoxes.filters.period.value = false;
            $scope.inputs.date.disabled = false;
            $scope.inputs.time.disabled = false;
            $scope.inputs.period.setDisabledValue(true);
          }
          $scope.inputs.apply.disabled = this.value ? false : true;
        }
      },
      period : {
        value : false,
        disabled : false,
        method : function() {
          if (this.value) {
            $scope.checkBoxes.filters.older.value = false;
            $scope.checkBoxes.filters.younger.value = false;
            $scope.inputs.date.disabled = true;
            $scope.inputs.time.disabled = true;
          }
          $scope.inputs.apply.disabled = this.value ? false : true;
          $scope.inputs.period.setDisabledValue(!this.value);
        }
      }
    }
  };

  $scope.month = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];

  $scope.toFullDate = function(str) {
    var date = '';
    date += str.getDate() < 10 ? '0' + str.getDate() : str.getDate();
    var month = '';
    month += (str.getMonth() + 1) < 10 ? '0' + (str.getMonth() + 1) : (str.getMonth() + 1);
    return {
      operate : str.getDate() + '-' + $scope.month[str.getMonth()] + '-' + str.getFullYear(),
      show : date + '.' + month + '.' + str.getFullYear()
    }
  };

  $scope.toFullTime = function (str) {
    var result =  str.getHours() + ':';
    result += (str.getMinutes() < 10) ? '0' + str.getMinutes() : str.getMinutes();
    return result;
  };

  $scope.getUnixTimeStamp = function(date, time) {
    return Date.parse($scope.toFullDate(date).operate + ' ' + $scope.toFullTime(time)).getTime()/1000;
  };

  $scope.apply = function() {
    $vkfilter.filters = {};

    function success () {
      $process.confirm_text += ' ?';
      $process.active_operation = 'delete_all';
      $vkfilter.context = 'wall';
      $user[$vkfilter.context].getWallPosts({
        offset : 0,
        count : 0
      }).success(function(res){
        $process.expected_value = res.response.count;
        $state.go('home.process');
      });
    }

    if ($scope.checkBoxes.filters.older.value || $scope.checkBoxes.filters.younger.value){
      if ($scope.inputs.date.value && $scope.inputs.time.value) {
        var fullDate = $scope.toFullDate($scope.inputs.date.value).show;
        var fullTime = $scope.toFullTime($scope.inputs.time.value);
        var filterValue = $scope.getUnixTimeStamp($scope.inputs.date.value, $scope.inputs.time.value);

        $process.confirm_text = $lang.wall.sure_date;

        if ($scope.checkBoxes.filters.older.value) {
          $process.confirm_text += ' ' + $lang.wall.before + ' ' + fullDate + ' ' + fullTime;
          $vkfilter.filters.date_older = filterValue;
        } else {
          $process.confirm_text += ' ' + $lang.wall.after + ' ' + fullDate + ' ' + fullTime;
          $vkfilter.filters.date_younger = filterValue;
        }

        success();

      } else {
        alert($lang.wall.alert_fill);
      }
    } else {
      if ($scope.inputs.period.valuesCorrect()) {
        var FullFromDate = $scope.toFullDate($scope.inputs.period.from_date.value).show;
        var FullFromTime = $scope.toFullTime($scope.inputs.period.from_time.value);
        var FullToDate = $scope.toFullDate($scope.inputs.period.to_date.value).show;
        var FullToTime = $scope.toFullTime($scope.inputs.period.to_time.value);
        var filterValue = {
          from : $scope.getUnixTimeStamp($scope.inputs.period.from_date.value, $scope.inputs.period.from_time.value),
          to : $scope.getUnixTimeStamp($scope.inputs.period.to_date.value, $scope.inputs.period.to_time.value)
        };
        $process.confirm_text = $lang.wall.sure_date + ' ' + $lang.wall.in_period + ': ' + FullFromDate + ' ' + FullFromTime +
            ' - ' + FullToDate + ' ' + FullToTime;
        $vkfilter.filters.period = filterValue;

        success();

      } else {
        alert($lang.wall.alert_fill);
      }
    }
  }

});
