/**
 * $user is an angular service that facilitate work 
 * with vk.com API. Contains main methods to perform  
 * profile operation on behalf of authorized user
 */

services.factory('$user', function ($http,
  $vk,
  $vkSession,
  $vkfilter,
  $errorHandler,
  $popup,
  $reqParams) {
  return {

    getFilteredMethod: function (_offset) {
      return this[$vkfilter.context].deleteAll({
        offset: _offset
      });
    },

    deleteObj: function (obj, params) {
      return this[$vkfilter.context].delete(obj, params);
    },

    /*
     * Recursive cleaning of user profile. 
     * vk.com limited the number of returned objects for 1 request.
     */

    deleteRec: function (obj, callback) {
      if (obj.scope.status === 'play') {
        var _this = this;
        if (obj.current_index < obj.end_index) {
          var current_object = obj.array[obj.current_index];
          if ($vkfilter.check(current_object)) {
            this.deleteObj(current_object).success(function (res) {
              if (res.response) {
                obj.scope.deleted++;
                obj.scope.current_index++;
                setTimeout(function () {
                  _this.deleteRec({
                    array: obj.array,
                    current_index: obj.current_index + 1,
                    end_index: obj.end_index,
                    scope: obj.scope
                  }, callback);
                }, $vkfilter.speed);
              } else {
                if (res.error) {
                  if (res.error.error_code == 14) {
                    $popup.showCaptcha({
                      scope: obj.scope,
                      img: res.error.captcha_img,
                      sid: res.error.captcha_sid
                    }, function (params) {
                      $reqParams.parameters = params;
                      _this.deleteRec(obj, callback);
                    });
                  } else {
                    $errorHandler.solve(res, function () {
                      setTimeout(function () {
                        _this.deleteRec({
                          array: obj.array,
                          current_index: obj.current_index,
                          end_index: obj.end_index,
                          scope: obj.scope
                        }, callback);
                      }, 1000);
                    })
                  }
                }
              }
            });
          } else {
            obj.scope.skip++;
            obj.scope.current_index++;
            _this.deleteRec({
              array: obj.array,
              current_index: obj.current_index + 1,
              end_index: obj.end_index,
              scope: obj.scope
            }, callback);
          }
        } else {
          this.getFilteredMethod(obj.scope.skip).success(function (res) {
            if (res.response.items.length > 0) {
              _this.deleteRec({
                array: res.response.items,
                current_index: 0,
                end_index: res.response.items.length,
                scope: obj.scope
              }, callback);
            } else {
              callback();
            }
          })
        }
      }
    },

    delete: function ($scope, callback) {
      var _this = this;
      _this.getFilteredMethod(0).success(function (res) {
        $scope.current_index = 0;
        $scope.deleted = 0;
        $scope.skip = 0;
        $scope.max_value = res.response.count;
        _this.deleteRec({
          array: res.response.items,
          current_index: 0,
          end_index: res.response.items.length,
          scope: $scope
        }, callback, {});
      })
    },

    /*
     * Makes request to get basic profile information of user 
     * Can be extended by adding new parameters to "fields" according 
     * to VK API. returns $promise. 
     */

    info: {
      getInfo: function () {
        return $vk.get({
          user_ids: $vkSession.access.user_id,
          fields: "photo_200_orig",
          name_case: "nom"
        }, 'users.get')
      }
    },


    wall: {
      post: function (message) {
        return $vk.get({
          owner_id: $vkSession.access.user_id,
          message: message,
          services: 'twitter,facebook',
          attachments: 'photo-112920019_421250245,http://vk.cc/4L70BG'
        }, 'wall.post')
      },

      repost: function (params) {
        return $vk.get(params, 'wall.repost');
      },

      getPostByID: function (id) {
        return $vk.get({
          posts: $vkSession.access.user_id + '_' + id
        }, 'wall.getById')
      },

      getWallPosts: function (params) {
        var _count = params.count ? params.count : $vk.maxWallPosts;
        var _filter = $vkfilter.filters.from ? $vkfilter.filters.from : "";

        if (params.filter) {
          _filter = params.filter;
        }

        return $vk.get({
          owner_id: $vkSession.access.user_id,
          offset: params.offset,
          filter: _filter,
          count: _count
        }, 'wall.get')
      },


      delete: function (post) {
        return $vk.get($reqParams.apply({
          owner_id: $vkSession.access.user_id,
          post_id: post.id
        }), 'wall.delete')
      },

      deleteAll: function (params) {
        return this.getWallPosts(params);
      }
    },

    communities: {
      getCommunityWall: function (community_id, offset, count) {
        return $vk.get({
          owner_id: community_id,
          offset: offset,
          count: count
        }, 'wall.get')
      },

      getCommunities: function (params) {
        var count = params.count ? params.count : $vk.maxCommunities;

        var filter = $vkfilter.filters ? $vkfilter.filters.community_type : "";

        if (params.filter) {
          filter = params.filter;
        }

        return $vk.get({
          owner_id: $vkSession.access.user_id,
          offset: params.offset,
          count: count,
          filter: filter,
          extended: 1,
          fields: 'members_count'
        }, 'groups.get')
      },

      delete: function (group) {
        return $vk.get($reqParams.apply({
          group_id: group.id
        }), 'groups.leave')
      },

      deleteAll: function (params) {
        return this.getCommunities(params);
      },

      isMember: function (group_id) {
        return $vk.get({
          group_id: group_id,
          user_id: $vkSession.access.user_id
        }, 'groups.isMember')
      },

      join: function (group_id) {
        return $vk.get({
          group_id: group_id
        }, 'groups.join')
      }
    },

    photos: {
      getPhotosFromAlbum: function (params) {
        var count = params.count ? params.count : $vk.maxPhotos;
        var request = params ? params : {};
        request.owner_id = $vkSession.access.user_id;
        request.count = count;
        return $vk.get(request, 'photos.get')
      },

      getAllPhotos: function (params) {
        var count = params.count ? params.count : $vk.maxPhotos;
        var request = params ? params : {};
        request.owner_id = $vkSession.access.user_id;
        request.extended = 1;
        request.count = count;
        request.no_service_albums = 0;
        return $vk.get(request, 'photos.getAll')
      },

      getAlbums: function (params) {
        var request = params ? params : {};
        request.owner_id = $vkSession.access.user_id;
        request.need_system = 1;
        request.need_covers = 1;
        request.photo_sizes = 1;
        return $vk.get(request, 'photos.getAlbums')
      },

      delete: function (photo) {
        return $vk.get($reqParams.apply({
          owner_id: $vkSession.access.user_id,
          photo_id: photo.id
        }), 'photos.delete')
      },

      deleteAll: function (params) {
        return this.getAllPhotos(params);
      }
    },

    photos_saved: {
      getPhotosFromAlbum: function (params) {
        var count = params.count ? params.count : $vk.maxPhotos;
        var request = params ? params : {};
        request.owner_id = $vkSession.access.user_id;
        request.count = count;
        return $vk.get(request, 'photos.get')
      },

      delete: function (photo) {
        return $vk.get($reqParams.apply({
          owner_id: $vkSession.access.user_id,
          photo_id: photo.id
        }), 'photos.delete')
      },

      deleteAll: function (params) {
        return this.getPhotosFromAlbum({
          album_id: 'saved'
        });
      }
    },

    audio: {
      getCount: function () {
        return $vk.get({
          owner_id: $vkSession.access.user_id
        }, 'audio.getCount')
      },

      getAudio: function (params) {
        return $vk.get({
          owner_id: $vkSession.access.user_id,
          offset: params.offset
        }, 'audio.get')
      },

      delete: function (audio) {
        return $vk.get($reqParams.apply({
          audio_id: audio.id,
          owner_id: $vkSession.access.user_id
        }), 'audio.delete')
      },

      deleteAll: function (params) {
        return this.getAudio(params);
      }
    },

    video: {
      getVideos: function (params) {
        var count = params.count ? params.count : $vk.maxVideos;
        return $vk.get({
          owner_id: $vkSession.access.user_id,
          count: count,
          offset: params.offset,
          extended: 0
        }, 'video.get')
      },

      delete: function (video) {
        return $vk.get($reqParams.apply({
          video_id: video.id,
          owner_id: video.owner_id,
          target_id: $vkSession.access.user_id
        }), 'video.delete')
      },


      deleteAll: function (params) {
        return this.getVideos(params);
      }
    },

    friends: {
      getFriends: function (params) {
        var count = params.count ? params.cout : $vk.maxFriends;
        return $vk.get({
          user_id: $vkSession.access.user_id,
          count: count,
          offset: params.offset,
          fields: 'city,last_seen'
        }, 'friends.get')
      },

      delete: function (friend) {
        return $vk.get($reqParams.apply({
          user_id: friend.id
        }), 'friends.delete')
      },

      deleteAll: function (params) {
        return this.getFriends(params);
      }
    },

    fav_photos: {

      getPhotos: function (params) {
        var req = params;
        req.count = $vk.maxFav;
        return $vk.get(req, 'fave.getPhotos');
      },

      delete: function (photo) {
        var req = {};
        req.item_id = photo.id;
        req.owner_id = photo.owner_id;
        req.type = "photo";
        return $vk.get($reqParams.apply(req), 'likes.delete')
      },

      deleteAll: function (params) {
        return this.getPhotos(params);
      }
    },

    fav_videos: {
      getVideos: function (params) {
        var req = params;
        req.count = $vk.maxFav;
        return $vk.get(req, 'fave.getVideos');
      },

      delete: function (video) {
        var req = {};
        req.item_id = video.id;
        req.owner_id = video.owner_id;
        req.type = "video";
        return $vk.get($reqParams.apply(req), 'likes.delete')
      },

      deleteAll: function (params) {
        return this.getVideos(params);
      }
    },

    fav_posts: {
      getPosts: function (params) {
        var req = params;
        req.count = $vk.maxFav;
        return $vk.get(req, 'fave.getPosts');
      },

      delete: function (post) {
        var req = {};
        req.item_id = post.id;
        req.owner_id = post.owner_id;
        req.type = "post";
        return $vk.get($reqParams.apply(req), 'likes.delete')
      },

      deleteAll: function (params) {
        return this.getPosts(params);
      }
    },

    likes: {
      add: function (params) {
        return $vk.get(params, 'likes.add');
      }
    }

  }
});